/****************************************************************************
 * $Rev:: 9497              $: Revision of last commit
 * $Author:: nxp33194       $: Author of last commit
 * $Date:: 2012-07-30 14:48#$: Date of last commit
 * $HeadURL: https://www.collabnet.nxp.com/svn/lprf_apps/Application_Notes/JN-AN-1133-ZigBee-PRO-Wireless-Lightswitch/Trunk/ZPC_Controller/Source/zpc_controller_node.c $
 ****************************************************************************
 * This software is owned by Jennic and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on Jennic products. You, and any third parties must reproduce
 * the copyright and warranty notice and any other legend of ownership on each
 * copy or partial copy of the software.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS". JENNIC MAKES NO WARRANTIES, WHETHER
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
 * ACCURACY OR LACK OF NEGLIGENCE. JENNIC SHALL NOT, IN ANY CIRCUMSTANCES,
 * BE LIABLE FOR ANY DAMAGES, INCLUDING, BUT NOT LIMITED TO, SPECIAL,
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON WHATSOEVER.
 *
 * Copyright Jennic Ltd 2007 - 2009. All rights reserved
 ****************************************************************************/

/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/

#include <jendefs.h>
#include "dbg.h"
#include "os.h"
#include "os_gen.h"
#include "pwrm.h"
#include "pdm.h"
#include "pdum_apl.h"
#include "zps_gen.h"
#include "zps_apl_af.h"
#include "app_buttons.h"
#include "ledcontrol.h"
#include "app_events.h"
#include "eventStrings.h"
/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/

#ifndef TRACE_APP
#define TRACE_APP TRUE
#endif

#define PERMIT_JOINING_TIME 5 //seconds

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/
typedef enum
{
    E_STARTUP,
    E_NETWORK_FORMATION,
    E_RUNNING
} teDeviceState;

/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/

PRIVATE void vStartup(void);
PRIVATE void vWaitForNetworkFormation(ZPS_tsAfEvent sStackEvent);
PRIVATE void vHandleStackEvent(ZPS_tsAfEvent sStackEvent);
PRIVATE void vHandleAppEvent(APP_tsEvent sAppEvent);


/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/
/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/

PRIVATE char *apcStateNames[] =
{
    "E_STARTUP",
    "E_NETWORK_FORMATION",
    "E_RUNNING"
};

PRIVATE teDeviceState s_eDeviceState;

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/

/**
 * Initialise the application
 */
PUBLIC void APP_vInitialiseNode(void)
{
    DBG_vPrintf(TRACE_APP, "APP_vInitialiseNode\r\n");

    /* Restoring context is not used in this example*/
    PDM_vDelete();

    /* Initialise ZBPro stack */
    ZPS_eAplAfInit();

    APP_vButtonInitialise();

    s_eDeviceState = E_STARTUP;
}

/**
 * Main task to handle all ZPS events for the coordinator node
 */
OS_TASK(APP_ZPC_Controller_Task)
{
    APP_tsEvent sAppEvent;
    ZPS_tsAfEvent sStackEvent;

    sStackEvent.eType = ZPS_EVENT_NONE;
    sAppEvent.eType = APP_E_EVENT_NONE;

    if (OS_E_OK != OS_eCollectMessage(APP_msgZpsEvents, &sStackEvent))
    {
        if (OS_E_OK == OS_eCollectMessage(APP_msgEvents, &sAppEvent))
        {
            DBG_vPrintf(TRACE_APP, "    Activation due to APP event: %s(%d) in state %s(%d) \r\n",
                    apcAPPEventStrings[sAppEvent.eType],sAppEvent.eType,
                    apcStateNames[s_eDeviceState],s_eDeviceState);
        }
    }
    else
    {
        DBG_vPrintf(TRACE_APP, "    Activation due to stack event: %s(%d) in state %s(%d) \r\n",
                apcZPSEventStrings[sStackEvent.eType],sStackEvent.eType,
                apcStateNames[s_eDeviceState],s_eDeviceState);

        if (ZPS_EVENT_APS_DATA_INDICATION == sStackEvent.eType)
        {
            DBG_vPrintf(TRACE_APP, "    Data Indication:\r\n");
            DBG_vPrintf(TRACE_APP, "        Profile :%x\r\n",sStackEvent.uEvent.sApsDataIndEvent.u16ProfileId);
            DBG_vPrintf(TRACE_APP, "        Cluster :%x\r\n",sStackEvent.uEvent.sApsDataIndEvent.u16ClusterId);
            DBG_vPrintf(TRACE_APP, "        EndPoint:%x\r\n",sStackEvent.uEvent.sApsDataIndEvent.u8DstEndpoint);
        }
    }

    switch (s_eDeviceState)
    {
        case E_STARTUP:
            vStartup();
            break;

        case E_NETWORK_FORMATION:
            vWaitForNetworkFormation(sStackEvent);
            break;

        case E_RUNNING:
            vHandleStackEvent(sStackEvent);
            vHandleAppEvent(sAppEvent);
            break;

        default:
            break;
    }

    /*
     * Global clean up to make sure any PDUs have been freed
     */
    if (ZPS_EVENT_APS_DATA_INDICATION == sStackEvent.eType)
    {
        PDUM_eAPduFreeAPduInstance(sStackEvent.uEvent.sApsDataIndEvent.hAPduInst);
    }
}



/****************************************************************************
 *
 * NAME: vStartup
 *
 * DESCRIPTION:
 * Initiates network formation
 *
 * RETURNS:
 * Sequence number
 *
 ****************************************************************************/
PRIVATE void vStartup(void)
{
    /* display the startup splash screen */
    DBG_vPrintf(TRACE_APP, "APP: Application startup\r\n");

    /* Stops Coordinator from sleeping ever - but does allow doze */
    PWRM_eStartActivity();

    vLedInitRfd();
    vLedControl(0, 1);
    vLedControl(1, 0);

    ZPS_teStatus eStatus = ZPS_eAplZdoStartStack();
    if (ZPS_E_SUCCESS == eStatus)
    {
        s_eDeviceState = E_NETWORK_FORMATION;
    }
    else
    {
        DBG_vPrintf(TRACE_APP, "APP: ZPS_eZdoStartStack() failed error %d\r\n", eStatus);
    }
}

/****************************************************************************
 *
 * NAME: vWaitForNetworkFormation
 *
 * DESCRIPTION:
 * Handles stack events during network formation
 *
 * PARAMETERS: Name         RW  Usage
 *             sStackEvent  R   Contains details of the stack event
 *
 * RETURNS:
 * Sequence number
 *
 ****************************************************************************/
PRIVATE void vWaitForNetworkFormation(ZPS_tsAfEvent sStackEvent)
{
    /* wait for network stack to start up as a coordinator */
    if (ZPS_EVENT_NONE != sStackEvent.eType)
    {
        if (ZPS_EVENT_NWK_STARTED == sStackEvent.eType)
        {
            DBG_vPrintf(TRACE_APP, "APP: Network Started\r\n");

            /* turn off joining */
            ZPS_eAplZdoPermitJoining(0);

            /* Turn off LEDs 0 and one to indicate button network started */
            vLedControl(0, 0);

            s_eDeviceState = E_RUNNING;
        }
    }
}

/****************************************************************************
 *
 * NAME: vHandleStackEvent
 *
 * DESCRIPTION:
 * Handles stack events after the Coordinator has started a network and is in
 * its running state
 *
 * PARAMETERS: Name         RW  Usage
 *             sStackEvent  R   Contains details of the stack event
 *
 * RETURNS:
 *
 ****************************************************************************/
PRIVATE void vHandleStackEvent(ZPS_tsAfEvent sStackEvent)
{
    if (ZPS_EVENT_NONE != sStackEvent.eType)
    {
        switch (sStackEvent.eType)
        {
            case ZPS_EVENT_APS_DATA_INDICATION:
                DBG_vPrintf(TRACE_APP, "APP: vCheckStackEvent: ZPS_EVENT_AF_DATA_INDICATION\r\n");
                break;

            case ZPS_EVENT_APS_DATA_CONFIRM:
                DBG_vPrintf(TRACE_APP, "APP: vCheckStackEvent: ZPS_EVENT_APS_DATA_CONFIRM Status %d, Address 0x%04x\r\n",
                            sStackEvent.uEvent.sApsDataConfirmEvent.u8Status,
                            sStackEvent.uEvent.sApsDataConfirmEvent.uDstAddr.u16Addr);
                break;

            case ZPS_EVENT_APS_DATA_ACK:
                DBG_vPrintf(TRACE_APP, "APP: vCheckStackEvent: ZPS_EVENT_APS_DATA_ACK Status %d, Address 0x%04x\r\n",
                            sStackEvent.uEvent.sApsDataAckEvent.u8Status,
                            sStackEvent.uEvent.sApsDataAckEvent.u16DstAddr);
                break;

            case ZPS_EVENT_NWK_NEW_NODE_HAS_JOINED:
                DBG_vPrintf(TRACE_APP, "APP: vCheckStackEvent: vCheckStackEvent: ZPS_EVENT_NEW_NODE_HAS_JOINED, Nwk Addr=0x%04x\r\n",
                            sStackEvent.uEvent.sNwkJoinIndicationEvent.u16NwkAddr);
                break;

            case ZPS_EVENT_NWK_LEAVE_INDICATION:
                DBG_vPrintf(TRACE_APP, "APP: vCheckStackEvent: ZPS_EVENT_LEAVE_INDICATION\r\n");
                break;

            case ZPS_EVENT_NWK_LEAVE_CONFIRM:
                DBG_vPrintf(TRACE_APP, "APP: vCheckStackEvent: ZPS_EVENT_LEAVE_CONFIRM\r\n");
                break;

            case ZPS_EVENT_NWK_STATUS_INDICATION:
                DBG_vPrintf(TRACE_APP, "APP: vCheckStackEvent: ZPS_EVENT_NWK_STATUS_INDICATION\r\n");
                break;

            case ZPS_EVENT_NWK_ROUTE_DISCOVERY_CONFIRM:
                DBG_vPrintf(TRACE_APP, "APP: vCheckStackEvent: ZPS_EVENT_ROUTE_DISCOVERY_CFM\r\n");
                break;

            case ZPS_EVENT_ERROR:
                DBG_vPrintf(TRACE_APP, "APP: vCheckStackEvent: ZPS_EVENT_ERROR\r\n");
                break;

            case ZPS_EVENT_NWK_POLL_CONFIRM:
                DBG_vPrintf(TRACE_APP, "APP: vCheckStackEvent: ZPS_EVENT_NEW_POLL_COMPLETE, status = %d\r\n",
                            sStackEvent.uEvent.sNwkPollConfirmEvent.u8Status );
                break;

            case ZPS_EVENT_NWK_JOINED_AS_ROUTER:
            case ZPS_EVENT_NWK_JOINED_AS_ENDDEVICE:
            case ZPS_EVENT_NWK_STARTED:
            case ZPS_EVENT_NWK_FAILED_TO_START:
            case ZPS_EVENT_NWK_FAILED_TO_JOIN:
            case ZPS_EVENT_NWK_DISCOVERY_COMPLETE:
                // Deliberate fall through
            default:
                DBG_vPrintf(TRACE_APP, "APP: vCheckStackEvent: unhandled event %d\r\n", sStackEvent.eType);
                break;
        }
    }
}

/****************************************************************************
 *
 * NAME: vHandleStackEvent
 *
 * DESCRIPTION:
 * Handles app events after the Coordinator has started a network and is in
 * its running state
 *
 * PARAMETERS: Name         RW  Usage
 *             sAppEvent  R   Contains details of the stack event
 *
 * RETURNS:
 *
 ****************************************************************************/
PRIVATE void vHandleAppEvent(APP_tsEvent sAppEvent)
{
    /* Button press starts the binding process */
    if (APP_E_EVENT_BUTTON_UP == sAppEvent.eType)
    {
        if (APP_E_BUTTONS_BUTTON_1 == sAppEvent.sButton.u8Button)
        {
            DBG_vPrintf(TRACE_APP, "Allow joining for %ds\r\n",PERMIT_JOINING_TIME);
            ZPS_eAplZdoPermitJoining(PERMIT_JOINING_TIME);
        }
        else
        {
            DBG_vPrintf(TRACE_APP,"Event ignored\r\n");
        }
    }
    else
    {
        DBG_vPrintf(TRACE_APP,"Event ignored\r\n");
    }
}




/****************************************************************************/
/***        Local Functions                                               ***/
/****************************************************************************/

/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
