/****************************************************************************
 * $Rev:: 9497              $: Revision of last commit
 * $Author:: nxp33194       $: Author of last commit
 * $Date:: 2012-07-30 14:48#$: Date of last commit
 * $HeadURL: https://www.collabnet.nxp.com/svn/lprf_apps/Application_Notes/JN-AN-1133-ZigBee-PRO-Wireless-Lightswitch/Trunk/ZPE_Switch/Source/zpe_switch_node.c $
 ****************************************************************************
 * This software is owned by Jennic and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on Jennic products. You, and any third parties must reproduce
 * the copyright and warranty notice and any other legend of ownership on each
 * copy or partial copy of the software.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS". JENNIC MAKES NO WARRANTIES, WHETHER
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
 * ACCURACY OR LACK OF NEGLIGENCE. JENNIC SHALL NOT, IN ANY CIRCUMSTANCES,
 * BE LIABLE FOR ANY DAMAGES, INCLUDING, BUT NOT LIMITED TO, SPECIAL,
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON WHATSOEVER.
 *
 * Copyright Jennic Ltd 2009. All rights reserved
 ****************************************************************************/

/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/

#include <jendefs.h>
#include <appapi.h>
#include "os.h"
#include "os_gen.h"
#include "pdum_apl.h"
#include "pdum_gen.h"
#include "pdm.h"
#include "dbg.h"
#include "pwrm.h"
#include "zps_gen.h"
#include "zps_apl_af.h"
#include "zps_apl_zdo.h"
#include "zps_apl_aib.h"
#include "zps_apl_zdp.h"

#include "LedControl.h"

#include "app_timer_driver.h"
#include "app_buttons.h"
#include "zpe_switch_node.h"

#include "eventStrings.h"
/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/

#ifndef TRACE_APP
#define TRACE_APP	TRUE
#endif

#define LED_TOGGLE_TIME APP_TIME_MS(200)
#define BIND_RESPONSE_WAIT_TIMER APP_TIME_MS(5000)


#define VALUE_ON                        0xFF
#define VALUE_OFF                       0x00
#define VALUE_TOGGLE                    0xF0
#define APDU_SWITCH_VALUES              0x00

#define ED_BIND_RESP					0x8020

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/
typedef enum
{
    APP_E_EVENT_NONE = 0,
    APP_E_EVENT_BUTTON_UP,
    APP_E_EVENT_BUTTON_DOWN
} APP_teEventType;

typedef struct
{
    uint8 u8Button;
} APP_teEventButton;

typedef struct
{
    APP_teEventType eType;
    union
    {
        APP_teEventButton sButton;
    };
} APP_tsEvent;
/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/
PRIVATE bool_t bBind(void);
PRIVATE void vToggleLed(uint8 u8LedNum);
PRIVATE void vSendToggleLightCmd(void);
/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

PRIVATE char *apcStateNames[] =
{
    "E_STARTUP",
    "E_WAIT_START",
    "E_NETWORK_DISCOVERY",
    "E_NETWORK_JOIN",
    "E_COMMISSIONING",
    "E_RESCAN",
    "E_BIND_WAIT",
    "E_RUNNING"
};
/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/

PRIVATE uint8 u8BindingSeqNum = 0;

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/

/**
 * Initialise the application
 */
PUBLIC void APP_vInitialiseNode(void)
{
    DBG_vPrintf(TRACE_APP, "APP_vInitialiseNode\r\n");

    /* Restoring context is not used in this example*/
    PDM_vDelete();

    /* Initialise ZBPro stack */
    ZPS_eAplAfInit();

    APP_vButtonInitialise();
}

/**
 * Main task to handle all ZPS events for the switch node
 */
OS_TASK(APP_ZPE_Switch_Task)
{
    APP_tsEvent sAppEvent;

    static enum
    {
        E_STARTUP,
        E_WAIT_START,
        E_NETWORK_DISCOVERY,
        E_NETWORK_JOIN,
        E_COMMISSIONING,
        E_RESCAN,
        E_BIND_WAIT,
        E_RUNNING
    } eNodeState = E_STARTUP;

    ZPS_tsAfEvent sStackEvent;

    sStackEvent.eType = ZPS_EVENT_NONE;
    sAppEvent.eType = APP_E_EVENT_NONE;


    if (OS_E_OK != OS_eCollectMessage(APP_msgZpsEvents, &sStackEvent))
    {
        if (OS_E_OK == OS_eCollectMessage(APP_msgEvents, &sAppEvent))
        {
            DBG_vPrintf(TRACE_APP, "    Activation due to APP event: %s(%d) in state %s(%d) \r\n",
                    apcAPPEventStrings[sAppEvent.eType],sAppEvent.eType,
                    apcStateNames[eNodeState],eNodeState);
        }
    }
    else
    {
        DBG_vPrintf(TRACE_APP, "    Activation due to stack event: %s(%d) in state %s(%d)\r\n",
                apcZPSEventStrings[sStackEvent.eType],sStackEvent.eType,
                apcStateNames[eNodeState],eNodeState);

        if (ZPS_EVENT_APS_DATA_INDICATION == sStackEvent.eType)
        {
            DBG_vPrintf(TRACE_APP, "    Data Indication:\r\n");
            DBG_vPrintf(TRACE_APP, "        Profile :%x\r\n",sStackEvent.uEvent.sApsDataIndEvent.u16ProfileId);
            DBG_vPrintf(TRACE_APP, "        Cluster :%x\r\n",sStackEvent.uEvent.sApsDataIndEvent.u16ClusterId);
            DBG_vPrintf(TRACE_APP, "        EndPoint:%x\r\n",sStackEvent.uEvent.sApsDataIndEvent.u8DstEndpoint);
        }
    }

    switch (eNodeState)
    {
        case E_STARTUP:
            /* Stay awake */
            PWRM_eStartActivity();
            vLedInitRfd();
            vLedControl(0, 0);
            vLedControl(1, 0);
            eNodeState = E_WAIT_START;
            OS_eActivateTask(APP_ZPE_Switch_Task);
        break;

        case E_WAIT_START:
        {
            if (OS_E_SWTIMER_RUNNING != OS_eGetSWTimerStatus(APP_RestartTimer))
            {
                vToggleLed(0);
                OS_eStartSWTimer(APP_RestartTimer, LED_TOGGLE_TIME, NULL);
            }

            /* Either button starts the joining process */
            if (APP_E_EVENT_BUTTON_UP == sAppEvent.eType)
            {
                if (APP_E_BUTTONS_BUTTON_1 == sAppEvent.sButton.u8Button)
                {

                    DBG_vPrintf(TRACE_APP, "Starting Join Process\r\n");
                    DBG_vPrintf(TRACE_APP, "Starting ZPS\r\n");
                    ZPS_teStatus eStatus = ZPS_eAplZdoStartStack();
                    if (ZPS_E_SUCCESS == eStatus)
                    {
                        vLedControl(0, 1);
                        DBG_vPrintf(TRACE_APP, "Stack started\r\n");
                        eNodeState = E_NETWORK_DISCOVERY;
                    }
                    else
                    {
                        DBG_vPrintf(TRACE_APP,
                                "ZPS_eAplZdoStartStack() failed error %d\r\n", eStatus);
                    }
                }
            }
            break;
        }

        case E_NETWORK_DISCOVERY:
        {
            if (ZPS_EVENT_NWK_DISCOVERY_COMPLETE == sStackEvent.eType)
            {
                DBG_vPrintf(TRACE_APP, "Network discovery complete\r\n");
                ZPS_tsAfNwkDiscoveryEvent *psNwkDiscoveryEvent =
                        &sStackEvent.uEvent.sNwkDiscoveryEvent;

                if (ZPS_E_SUCCESS == psNwkDiscoveryEvent->eStatus)
                {
                    DBG_vPrintf(
                            TRACE_APP,
                            "Found %d networks\r\n",
                            psNwkDiscoveryEvent->u8NetworkCount);
                }
                else
                {
                    DBG_vPrintf(
                            TRACE_APP,
                            "Network discovery failed with error %d\r\n",
                            psNwkDiscoveryEvent->eStatus);
                }

                if (0 == psNwkDiscoveryEvent->u8NetworkCount ||
                        0xff == psNwkDiscoveryEvent->u8SelectedNetwork)
                {
                    if (0 != psNwkDiscoveryEvent->u32UnscannedChannels)
                    {
                        /* continue to look for networks on un-scanned channels */
                        DBG_vPrintf(TRACE_APP,"No networks found, continue scanning ...\r\n");
                        ZPS_eAplZdoDiscoverNetworks(psNwkDiscoveryEvent->u32UnscannedChannels);
                    }
                    else
                    {
                        DBG_vPrintf(TRACE_APP,"Exhausted channels to scan\r\n");
                        eNodeState = E_WAIT_START;
                        OS_eActivateTask(APP_ZPE_Switch_Task);
                    }
                }
                else
                {
                    ZPS_tsNwkNetworkDescr *psNwkDescr
                            = &psNwkDiscoveryEvent->asNwkDescriptors[psNwkDiscoveryEvent->u8SelectedNetwork];
                    ZPS_teStatus eStatus;

                    DBG_vPrintf(TRACE_APP, "Unscanned channels %08x\r\n",
                            psNwkDiscoveryEvent->u32UnscannedChannels);

                    DBG_vPrintf(TRACE_APP, "\tExt PAN ID = %016llx\r\n", psNwkDescr->u64ExtPanId);
                    DBG_vPrintf(TRACE_APP, "\tChannel = %d\r\n", psNwkDescr->u8LogicalChan);
                    DBG_vPrintf(TRACE_APP, "\tStack Profile = %d\r\n",psNwkDescr->u8StackProfile);
                    DBG_vPrintf(TRACE_APP, "\tZigbee Version = %d\r\n", psNwkDescr->u8ZigBeeVersion);
                    DBG_vPrintf(TRACE_APP, "\tPermit Joining = %d\r\n", psNwkDescr->u8PermitJoining);
                    DBG_vPrintf(TRACE_APP, "\tRouter Capacity = %d\r\n", psNwkDescr->u8RouterCapacity);
                    DBG_vPrintf(TRACE_APP, "\tEnd Device Capacity = %d\r\n", psNwkDescr->u8EndDeviceCapacity);

                    eStatus = ZPS_eAplZdoJoinNetwork(psNwkDescr);

                    if (ZPS_E_SUCCESS == eStatus)
                    {
                        DBG_vPrintf(TRACE_APP, "Joining network\r\n");
                        eNodeState = E_NETWORK_JOIN;
                    }
                    else
                    {
                        /* start scan again */
                        DBG_vPrintf(TRACE_APP,
                                "Failed to join network reason = %02x\r\n",
                                eStatus);
                        eNodeState = E_WAIT_START;
                        OS_eActivateTask(APP_ZPE_Switch_Task);
                    }
                }
            }
            else if (ZPS_EVENT_NWK_FAILED_TO_JOIN == sStackEvent.eType)
            {
            	DBG_vPrintf(TRACE_APP, "Network failed to join error = 0x%x\r\n",
            			sStackEvent.uEvent.sNwkJoinFailedEvent.u8Status);

                eNodeState = E_WAIT_START;
                OS_eActivateTask(APP_ZPE_Switch_Task);
            }
            else if (ZPS_EVENT_NWK_JOINED_AS_ENDDEVICE == sStackEvent.eType)
            {
            	DBG_vPrintf(TRACE_APP, "Node joined network, Addr: 0x%04x\n",
            	            sStackEvent.uEvent.sNwkJoinedEvent.u16Addr);

                eNodeState = E_COMMISSIONING;
                OS_eActivateTask(APP_ZPE_Switch_Task);
            }
            else
            {
                DBG_vPrintf(TRACE_APP,"Event ignored\r\n");
            }
            break;
        }

        case E_NETWORK_JOIN:
        {
            if (ZPS_EVENT_NWK_JOINED_AS_ENDDEVICE == sStackEvent.eType)
            {
                DBG_vPrintf(TRACE_APP, "Node joined network, Addr: 0x%04x\n",
                            sStackEvent.uEvent.sNwkJoinedEvent.u16Addr);

                eNodeState = E_COMMISSIONING;
                OS_eActivateTask(APP_ZPE_Switch_Task);
            }
            else if (ZPS_EVENT_NWK_FAILED_TO_JOIN == sStackEvent.eType)
            {
                DBG_vPrintf(TRACE_APP,
                            "Node failed to join network. Retrying ...\r\n");
                eNodeState = E_WAIT_START;
                OS_eActivateTask(APP_ZPE_Switch_Task);
            }
            else
            {
                DBG_vPrintf(TRACE_APP,"Event ignored\r\n");
            }
            break;
        }

        case E_RESCAN:
            /* restart scan with configured channel mask */
            DBG_vPrintf(TRACE_APP, "Rescanning with %08x\r\n",ZPS_psAplAibGetAib()->apsChannelMask);
            ZPS_eAplZdoDiscoverNetworks(ZPS_psAplAibGetAib()->apsChannelMask);
            eNodeState = E_NETWORK_DISCOVERY;
        break;

            /* Commissioning */
        case E_COMMISSIONING:
            vLedControl(0, 0);
            /* Button press starts the binding process */
            if (APP_E_EVENT_BUTTON_UP == sAppEvent.eType)
            {
                if (APP_E_BUTTONS_BUTTON_2 == sAppEvent.sButton.u8Button)
                {
                    DBG_vPrintf(TRACE_APP, "Button pressed\r\n");
                    if (bBind() == TRUE)
                    {
                        eNodeState = E_BIND_WAIT;
                        OS_eActivateTask(APP_ZPE_Switch_Task);

                        OS_eStopSWTimer(APP_RestartTimer);
                        OS_eStartSWTimer(APP_RestartTimer,
                                         BIND_RESPONSE_WAIT_TIMER,
                                         NULL);

                    }
                    else
                    {
                        DBG_vPrintf(TRACE_APP, "Send bind failed\r\n");
                    }
                }
                else
                {
                    DBG_vPrintf(TRACE_APP,"Event ignored\r\n");
                }
            }
            else
            {
                if (OS_E_SWTIMER_RUNNING != OS_eGetSWTimerStatus(APP_RestartTimer))
                {
                    vToggleLed(1);
                    OS_eStartSWTimer(APP_RestartTimer, LED_TOGGLE_TIME, NULL);
                }
            }
        break;

        case E_BIND_WAIT:
        {
            vLedControl(0, 0);
            vLedControl(1, 1);

            if (ZPS_EVENT_NONE != sStackEvent.eType)
            {
                if (ZPS_EVENT_APS_ZDP_REQUEST_RESPONSE == sStackEvent.eType)
    			{
    				if (ED_BIND_RESP == sStackEvent.uEvent.sApsZdpEvent.u16ClusterId)
    				{
    					if(sStackEvent.uEvent.sApsZdpEvent.uZdpData.sBindRsp.u8Status !=  ZPS_E_SUCCESS)
    					{
                            DBG_vPrintf(TRACE_APP, "Binding Response Failed\r\n");
                            eNodeState = E_COMMISSIONING;
                            OS_eActivateTask(APP_ZPE_Switch_Task);
                        }
                        else
                        {
                            DBG_vPrintf(TRACE_APP, "Binding Response OK\r\n");
                            eNodeState = E_RUNNING;
                            OS_eActivateTask(APP_ZPE_Switch_Task);
                        }
                    }
                    else
                    {
                    	DBG_vPrintf(TRACE_APP,"ZDP Event ignored, cluster ID: 0x%x\r\n", sStackEvent.uEvent.sApsZdpEvent.u16ClusterId);
                    }
                }
                else
                {
                    DBG_vPrintf(TRACE_APP,"Event ignored\r\n");
                }
            }
            else if (OS_E_SWTIMER_RUNNING
                    != OS_eGetSWTimerStatus(APP_RestartTimer))
            {
                DBG_vPrintf(TRACE_APP, "No bind response\r\n");
                eNodeState = E_COMMISSIONING;
                OS_eActivateTask(APP_ZPE_Switch_Task);
            }
        }
        break;

        case E_RUNNING:
            vLedControl(0, 0);
            vLedControl(1, 0);
            /* check for button presses */
            if (APP_E_EVENT_BUTTON_UP == sAppEvent.eType
                    && APP_E_BUTTONS_BUTTON_1 == sAppEvent.sButton.u8Button)
            {
                vSendToggleLightCmd();
            }
            else
            {
                DBG_vPrintf(TRACE_APP,"Event ignored\r\n");
            }
        break;
    }

    /*
     * Global clean up to make sure any PDUs have been freed
     */
    if (ZPS_EVENT_APS_DATA_INDICATION == sStackEvent.eType)
    {
        PDUM_eAPduFreeAPduInstance(sStackEvent.uEvent.sApsDataIndEvent.hAPduInst);
    }

}

/****************************************************************************/
/***        Local Functions                                               ***/
/****************************************************************************/


/**
 * Called to send the end device bind request.
 */
PRIVATE bool_t bBind(void)
{
    bool_t bStatus = FALSE;

    PDUM_thAPduInstance hAPduInst =
            PDUM_hAPduAllocateAPduInstance(apduApplication);

    if (PDUM_INVALID_HANDLE != hAPduInst)
    {
        ZPS_tsAplZdpEndDeviceBindReq sEndDeviceBindReq;

        uint16 au16OutClusters[] =
        { HOME_CONTROLS_ON_OFF_SWITCH_ACTIONS_CLUSTER_ID };

        sEndDeviceBindReq.u16BindingTarget
                = ZPS_u16NwkNibGetNwkAddr(ZPS_pvAplZdoGetNwkHandle());
        sEndDeviceBindReq.u64SrcIeeeAddress
                = ZPS_u64AplZdoGetIeeeAddr();

        DBG_vPrintf(TRACE_APP,"Target MAC:0x%llx Nwk:%d\r\n",
                    sEndDeviceBindReq.u64SrcIeeeAddress,
                    sEndDeviceBindReq.u16BindingTarget);

        sEndDeviceBindReq.u8SrcEndpoint = 1;
        sEndDeviceBindReq.u16ProfileId = HOME_CONTROLS_PROFILE_ID;
        sEndDeviceBindReq.u8NumInClusters = 0;
        sEndDeviceBindReq.pu16InClusterList = NULL;
        sEndDeviceBindReq.u8NumOutClusters = 1;
        sEndDeviceBindReq.pu16OutClusterList = au16OutClusters;

        ZPS_teStatus teStatus =
                ZPS_eAplZdpEndDeviceBindRequest(hAPduInst,
                                                &u8BindingSeqNum,
                                                &sEndDeviceBindReq);

        if (teStatus)
        {
            DBG_vPrintf(TRACE_APP, "Error, Bind Failed: %x\r\n", teStatus );
        }
        else
        {
            DBG_vPrintf(TRACE_APP, "Binding Request OK, seq num %d\r\n", u8BindingSeqNum );
            bStatus = TRUE;
        }
    }
    else
    {
        DBG_vPrintf(TRACE_APP, "Error, no free PDUs\r\n");
    }

    return bStatus;
}

/**
 * Toggles the specified LED
 */
PRIVATE void vToggleLed(uint8 u8LedNum)
{
    static bool_t bLedStates[2] =
    { FALSE, FALSE };

    bLedStates[u8LedNum] = !bLedStates[u8LedNum];
    vLedControl(u8LedNum,bLedStates[u8LedNum]);
}

/**
 * Send packet to the bound light node to toggle the
 * light.
 */
PRIVATE void vSendToggleLightCmd(void)
{
    uint8 u8ButtonVal = VALUE_TOGGLE;

    PDUM_thAPduInstance hAPduInst = PDUM_hAPduAllocateAPduInstance(apduApplication);

    if (PDUM_INVALID_HANDLE != hAPduInst)
    {
        PDUM_u16APduInstanceWriteNBO(hAPduInst, APDU_SWITCH_VALUES,
                                     "a\x01", &u8ButtonVal);

        PDUM_eAPduInstanceSetPayloadSize(hAPduInst, 1);

        ZPS_teStatus teStatus = ZPS_eAplAfBoundDataReq(hAPduInst,
                HOME_CONTROLS_ON_OFF_SWITCH_ACTIONS_CLUSTER_ID,
                ZPE_SWITCH_LIGHT_ENDPOINT, ZPS_E_APL_AF_UNSECURE, 0,
                NULL);

        if (teStatus)
        {
            DBG_vPrintf(TRACE_APP, "Bound Send Failed: %x\r\n", teStatus );
        }
        else
        {
            DBG_vPrintf(TRACE_APP, "Bind Send Success\n");
        }

        DBG_vPrintf(TRACE_APP, "Sent Toggle Event\r\n");
    }
    else
    {
        DBG_vPrintf(TRACE_APP, "Failed to get APDU\r\n");
    }
}




/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
