/****************************************************************************
 * $Rev:: 3438              $: Revision of last commit
 * $Author:: snich          $: Author of last commit
 * $Date:: 2009-06-30 13:31#$: Date of last commit
 * $HeadURL: https://www.collabnet.nxp.com/svn/lprf_apps/Application_Notes/JN-AN-1133-ZigBee-PRO-Wireless-Lightswitch/Trunk/ZPR_Light/Source/light_switch_handler.c $
 ****************************************************************************
 * This software is owned by Jennic and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on Jennic products. You, and any third parties must reproduce
 * the copyright and warranty notice and any other legend of ownership on each
 * copy or partial copy of the software.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS". JENNIC MAKES NO WARRANTIES, WHETHER
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
 * ACCURACY OR LACK OF NEGLIGENCE. JENNIC SHALL NOT, IN ANY CIRCUMSTANCES,
 * BE LIABLE FOR ANY DAMAGES, INCLUDING, BUT NOT LIMITED TO, SPECIAL,
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON WHATSOEVER.
 *
 * Copyright Jennic Ltd 2007 - 2009. All rights reserved
 ****************************************************************************/

/****************************************************************************/
/*          Include files                                                   */
/****************************************************************************/

#include <jendefs.h>
#include "os.h"
#include "os_gen.h"
#include "pdum_apl.h"
#include "pdum_gen.h"
#include "dbg.h"

#include "zps_gen.h"
#include "zps_apl.h"
#include "zps_apl_af.h"
#include "eventStrings.h"
#include "ledcontrol.h"
/****************************************************************************/
/*          Macro Definitions                                               */
/****************************************************************************/
#ifndef TRACE_APP
#define TRACE_APP TRUE
#endif

#define VALUE_ON                        0xFF
#define VALUE_OFF                       0x00
#define VALUE_TOGGLE                    0xF0
#define APDU_SWITCH_VALUES              0x00

/****************************************************************************/
/***        Type Definitions                                                */
/****************************************************************************/

/****************************************************************************/
/*          Local Function Prototypes                                       */
/****************************************************************************/

/****************************************************************************/
/*          Exported Variables                                              */
/****************************************************************************/

/****************************************************************************/
/*          Local Variables                                                 */
/****************************************************************************/

/****************************************************************************/
/*          Exported Functions                                              */
/****************************************************************************/

/**
 * Task that handles all data that is received on end point 1
 */
OS_TASK(APP_LightHandler_Task)
{
    ZPS_tsAfEvent sStackEvent;

    sStackEvent.eType = ZPS_EVENT_NONE;

    DBG_vPrintf(TRACE_APP, "APP_LightHandler_Task:Task Activated \r\n");

    if (OS_E_OK == OS_eCollectMessage(APP_msgZPSEndPoint1, &sStackEvent))
    {
        DBG_vPrintf(TRACE_APP, "APP_LightHandler_Task Activation due to stack event: %s(%d)\r\n",
                apcZPSEventStrings[sStackEvent.eType],sStackEvent.eType);

        if (ZPS_EVENT_APS_DATA_INDICATION == sStackEvent.eType)
        {
            DBG_vPrintf(TRACE_APP, "    Data Indication:\r\n");
            DBG_vPrintf(TRACE_APP, "        Profile :%x\r\n",sStackEvent.uEvent.sApsDataIndEvent.u16ProfileId);
            DBG_vPrintf(TRACE_APP, "        Cluster :%x\r\n",sStackEvent.uEvent.sApsDataIndEvent.u16ClusterId);
            DBG_vPrintf(TRACE_APP, "        EndPoint:%x\r\n",sStackEvent.uEvent.sApsDataIndEvent.u8DstEndpoint);
        }
    }

    if (ZPS_EVENT_APS_DATA_INDICATION == sStackEvent.eType)
    {
        uint8 u8SwitchVal = 0;

        if (sStackEvent.uEvent.sApsDataIndEvent.u16ProfileId == HOME_CONTROLS_PROFILE_ID
                && sStackEvent.uEvent.sApsDataIndEvent.u16ClusterId == HOME_CONTROLS_ON_OFF_SWITCH_ACTIONS_CLUSTER_ID)
        {
            if (0 != PDUM_u16APduInstanceReadNBO(sStackEvent.uEvent.sApsDataIndEvent.hAPduInst,
                                                   0,
                                                   "a\x01",
                                                   &u8SwitchVal))
            {

                DBG_vPrintf(TRACE_APP, "button val %d\r\n", u8SwitchVal );
                static bool bLedState = FALSE;

                if (u8SwitchVal == VALUE_OFF)
                {
                    vLedControl(0, 0);
                    bLedState = FALSE;
                }
                else if (u8SwitchVal == VALUE_ON)
                {
                    vLedControl(0, 1);
                    bLedState = TRUE;
                }
                else if (u8SwitchVal == VALUE_TOGGLE)
                {
                    if (bLedState)
                    {
                        vLedControl(0, 0);
                        bLedState = FALSE;
                    }
                    else
                    {
                        vLedControl(0, 1);
                        bLedState = TRUE;
                    }
                }
            }
            else
            {
                DBG_vPrintf(TRACE_APP, "APP - APDU read failed to read any data.\r\n");
            }
        }
        else
        {
            DBG_vPrintf(TRACE_APP, "Not for me.\r\n");
        }
    }


    /* Generic clean up function to ensure any APDUs are freed */
    if (ZPS_EVENT_APS_DATA_INDICATION == sStackEvent.eType)
    {
        PDUM_eAPduFreeAPduInstance(sStackEvent.uEvent.sApsDataIndEvent.hAPduInst);
    }
}

/****************************************************************************/
/***        Local Functions                                                 */
/****************************************************************************/

/****************************************************************************/
/*          END OF FILE                                                     */
/****************************************************************************/
