/****************************************************************************
 * $Rev:: 9497              $: Revision of last commit
 * $Author:: nxp33194       $: Author of last commit
 * $Date:: 2012-07-30 14:48#$: Date of last commit
 * $HeadURL: https://www.collabnet.nxp.com/svn/lprf_apps/Application_Notes/JN-AN-1133-ZigBee-PRO-Wireless-Lightswitch/Trunk/ZPR_Light/Source/zpr_light_node.c $
 ****************************************************************************
 * This software is owned by Jennic and/or its supplier and is protected
 * under applicable copyright laws. All rights are reserved. We grant You,
 * and any third parties, a license to use this software solely and
 * exclusively on Jennic products. You, and any third parties must reproduce
 * the copyright and warranty notice and any other legend of ownership on each
 * copy or partial copy of the software.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS". JENNIC MAKES NO WARRANTIES, WHETHER
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
 * ACCURACY OR LACK OF NEGLIGENCE. JENNIC SHALL NOT, IN ANY CIRCUMSTANCES,
 * BE LIABLE FOR ANY DAMAGES, INCLUDING, BUT NOT LIMITED TO, SPECIAL,
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON WHATSOEVER.
 *
 * Copyright Jennic Ltd 2007 - 2009. All rights reserved
 ****************************************************************************/
/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/
#include <jendefs.h>
#include "dbg.h"
#include "os.h"
#include "os_gen.h"
#include "pdum_apl.h"
#include "pdum_gen.h"
#include "pwrm.h"
#include "pdm.h"
#include "pdum_apl.h"
#include "pdum_gen.h"
#include "zps_gen.h"
#include "zps_apl.h"
#include "zps_apl_af.h"
#include "zps_apl_zdo.h"
#include "zps_apl_zdp.h"
#include "zps_apl_aib.h"
#include "app_buttons.h"
#include "app_timer_driver.h"
#include "ledcontrol.h"
#include "app_events.h"
#include "eventStrings.h"
/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/
#ifndef TRACE_APP
#define TRACE_APP TRUE
#endif

#define PERMIT_JOINING_TIME 5
#define LED_TOGGLE_TIME APP_TIME_MS(200)
#define BIND_RESPONSE_WAIT_TIMER APP_TIME_MS(5000)

#define ED_BIND_RESP						0x8020

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/
/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/
PRIVATE bool_t bBind(void);
PRIVATE void vToggleLed(uint8 u8LedNum);
/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/
/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/
PRIVATE uint8 u8BindingSeqNum = 0;
PRIVATE bool_t bBound;
PRIVATE char *apcStateNames[] =
{
    "E_STARTUP",
    "E_WAIT_START",
    "E_NETWORK_DISCOVERY",
    "E_NETWORK_JOIN",
    "E_RESCAN",
    "E_BIND_WAIT",
    "E_RUNNING"
};
/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/

/**
 * Initialises the node
 */
PUBLIC void APP_vInitialiseNode(void)
{
	DBG_vPrintf(TRACE_APP, "APP_vInitialiseNode\r\n");

    /* Restoring context is not used in this example*/
    PDM_vDelete();

    /* Initialise ZBPro stack */
    ZPS_eAplAfInit();

    APP_vButtonInitialise();
}


/**
 * Main task to handle all ZPS events for the light node
 */
OS_TASK(APP_ZPR_Light_Task)
{
    static enum
    {
        E_STARTUP,
        E_WAIT_START,
        E_NETWORK_DISCOVERY,
        E_NETWORK_JOIN,
        E_RESCAN,
        E_BIND_WAIT,
        E_RUNNING
    } eNodeState = E_STARTUP;

    APP_tsEvent sAppEvent;
    ZPS_tsAfEvent sStackEvent;

    sStackEvent.eType = ZPS_EVENT_NONE;
    sAppEvent.eType = APP_E_EVENT_NONE;

    if (OS_E_OK != OS_eCollectMessage(APP_msgZpsEvents, &sStackEvent))
    {
        if (OS_E_OK == OS_eCollectMessage(APP_msgEvents, &sAppEvent))
        {
            DBG_vPrintf(TRACE_APP, "    Activation due to APP event: %s(%d) in state %s(%d) \r\n",
                    apcAPPEventStrings[sAppEvent.eType],sAppEvent.eType,
                    apcStateNames[eNodeState],eNodeState);
        }
    }
    else
    {
        DBG_vPrintf(TRACE_APP, "    Activation due to stack event: %s(%d) in state %s(%d) \r\n",
                apcZPSEventStrings[sStackEvent.eType],sStackEvent.eType,
                apcStateNames[eNodeState],eNodeState);

        if (ZPS_EVENT_APS_DATA_INDICATION == sStackEvent.eType)
        {
            DBG_vPrintf(TRACE_APP, "    Data Indication:\r\n");
            DBG_vPrintf(TRACE_APP, "        Profile :%x\r\n",sStackEvent.uEvent.sApsDataIndEvent.u16ProfileId);
            DBG_vPrintf(TRACE_APP, "        Cluster :%x\r\n",sStackEvent.uEvent.sApsDataIndEvent.u16ClusterId);
            DBG_vPrintf(TRACE_APP, "        EndPoint:%x\r\n",sStackEvent.uEvent.sApsDataIndEvent.u8DstEndpoint);
        }

    }

    if (ZPS_EVENT_ERROR == sStackEvent.eType)
    {
        ZPS_tsAfErrorEvent *psErrEvt = &sStackEvent.uEvent.sAfErrorEvent;
        DBG_vPrintf(TRACE_APP,"Stack Err: %d\r\n",psErrEvt->eError);
    }

    switch (eNodeState)
    {

        case E_STARTUP:
            /* Stay awake */
            PWRM_eStartActivity();
            vLedInitRfd();
            vLedControl(0, 0);
            vLedControl(1, 0);
            eNodeState = E_WAIT_START;
            OS_eActivateTask(APP_ZPR_Light_Task);
            bBound = FALSE;
        break;

        case E_WAIT_START:
        {
            if (OS_E_SWTIMER_RUNNING != OS_eGetSWTimerStatus(APP_RestartTimer))
            {
                vToggleLed(0);
                OS_eStartSWTimer(APP_RestartTimer, LED_TOGGLE_TIME, NULL);
            }

            /* Either button starts the joining process */
            if (APP_E_EVENT_BUTTON_UP == sAppEvent.eType)
            {
                if (APP_E_BUTTONS_BUTTON_1 == sAppEvent.sButton.u8Button)
                {
                    /* turn off led1 to show starting join process */
                    vLedControl(1, 0);
                    DBG_vPrintf(TRACE_APP, "Starting Join Process\r\n");
                    DBG_vPrintf(TRACE_APP, "Starting ZPS\r\n");
                    ZPS_teStatus eStatus = ZPS_eAplZdoStartStack();
                    if (ZPS_E_SUCCESS == eStatus)
                    {
                        /* turn off led1 to show starting join process */
                        vLedControl(1, 0);
                        DBG_vPrintf(TRACE_APP, "Stack started\r\n");
                        eNodeState = E_NETWORK_DISCOVERY;
                    }
                    else
                    {
                        DBG_vPrintf(TRACE_APP,
                                "ZPS_eAplZdoStartStack() failed error %d\r\n", eStatus);
                    }
                }
            }
            break;
        }

        case E_NETWORK_DISCOVERY:
        {
            if (ZPS_EVENT_NWK_DISCOVERY_COMPLETE == sStackEvent.eType)
            {
                DBG_vPrintf(TRACE_APP, "Network discovery complete\r\n");
                ZPS_tsAfNwkDiscoveryEvent *psNwkDiscoveryEvent =
                        &sStackEvent.uEvent.sNwkDiscoveryEvent;

                if (ZPS_E_SUCCESS == psNwkDiscoveryEvent->eStatus)
                {
                    DBG_vPrintf(
                            TRACE_APP,
                            "Found %d networks\r\n",
                            psNwkDiscoveryEvent->u8NetworkCount);
                }
                else
                {
                    DBG_vPrintf(
                            TRACE_APP,
                            "Network discovery failed with error %d\r\n",
                            psNwkDiscoveryEvent->eStatus);
                }

                if (0 == psNwkDiscoveryEvent->u8NetworkCount ||
                        0xff == psNwkDiscoveryEvent->u8SelectedNetwork)
                {
                    if (0 != psNwkDiscoveryEvent->u32UnscannedChannels)
                    {
                        /* continue to look for networks on un-scanned channels */
                        DBG_vPrintf(TRACE_APP,"No networks found, continue scanning ...\r\n");
                        ZPS_eAplZdoDiscoverNetworks(psNwkDiscoveryEvent->u32UnscannedChannels);
                    }
                    else
                    {
                        DBG_vPrintf(TRACE_APP,"Exhausted channels to scan\r\n");
                        eNodeState = E_WAIT_START;
                        OS_eActivateTask(APP_ZPR_Light_Task);
                    }
                }
                else
                {
                    ZPS_tsNwkNetworkDescr
                            *psNwkDescr = &psNwkDiscoveryEvent->asNwkDescriptors[psNwkDiscoveryEvent->u8SelectedNetwork];
                    ZPS_teStatus eStatus;

                    DBG_vPrintf(TRACE_APP, "Unscanned channels %08x\r\n",
                            psNwkDiscoveryEvent->u32UnscannedChannels);

                    DBG_vPrintf(TRACE_APP, "\tExt PAN ID = %016llx\r\n", psNwkDescr->u64ExtPanId);
                    DBG_vPrintf(TRACE_APP, "\tChannel = %d\r\n", psNwkDescr->u8LogicalChan);
                    DBG_vPrintf(TRACE_APP, "\tStack Profile = %d\r\n", psNwkDescr->u8StackProfile);
                    DBG_vPrintf(TRACE_APP, "\tZigbee Version = %d\r\n", psNwkDescr->u8ZigBeeVersion);
                    DBG_vPrintf(TRACE_APP, "\tPermit Joining = %d\r\n", psNwkDescr->u8PermitJoining);
                    DBG_vPrintf(TRACE_APP, "\tRouter Capacity = %d\r\n", psNwkDescr->u8RouterCapacity);
                    DBG_vPrintf(TRACE_APP, "\tEnd Device Capacity = %d\r\n", psNwkDescr->u8EndDeviceCapacity);

                    eStatus = ZPS_eAplZdoJoinNetwork(psNwkDescr);

                    if (ZPS_E_SUCCESS == eStatus)
                    {
                        DBG_vPrintf(TRACE_APP, "Joining network\r\n");
                        eNodeState = E_NETWORK_JOIN;
                    }
                    else
                    {
                        /* start scan again */
                        DBG_vPrintf(TRACE_APP,
                                "Failed to join network reason = %02x\r\n",
                                eStatus);
                        eNodeState = E_WAIT_START;
                        OS_eActivateTask(APP_ZPR_Light_Task);
                    }
                }

            }
            else if (ZPS_EVENT_NWK_FAILED_TO_JOIN == sStackEvent.eType)
            {
                DBG_vPrintf(TRACE_APP, "Network failed to join error = 0x%x\r\n",
                		sStackEvent.uEvent.sNwkJoinFailedEvent.u8Status);
                eNodeState = E_WAIT_START;
                OS_eActivateTask(APP_ZPR_Light_Task);
            }
            else if (ZPS_EVENT_NWK_JOINED_AS_ROUTER == sStackEvent.eType)
            {
            	DBG_vPrintf(TRACE_APP, "Node joined network, Addr: 0x%04x\n",
            	                sStackEvent.uEvent.sNwkJoinedEvent.u16Addr);

                /* Turn off led 0 to show joined */
                vLedControl(0, 0);
                OS_eStopSWTimer(APP_RestartTimer);
                eNodeState = E_RUNNING;
                OS_eActivateTask(APP_ZPR_Light_Task);
            }
            else
            {
                DBG_vPrintf(TRACE_APP,"Event ignored\r\n");
            }
            break;
        }

        case E_NETWORK_JOIN:
        {
            if (ZPS_EVENT_NWK_JOINED_AS_ROUTER == sStackEvent.eType)
            {
                DBG_vPrintf(TRACE_APP, "Node joined network, Addr: 0x%04x\n",
                                sStackEvent.uEvent.sNwkJoinedEvent.u16Addr);
                /* Turn off led 0 to show joined */
                vLedControl(0, 0);
                OS_eStopSWTimer(APP_RestartTimer);
                eNodeState = E_RUNNING;
                OS_eActivateTask(APP_ZPR_Light_Task);
            }
            else if (ZPS_EVENT_NWK_FAILED_TO_JOIN == sStackEvent.eType)
            {
                DBG_vPrintf(TRACE_APP,
                        "Node failed to join network. Retrying ...\r\n");
                eNodeState = E_WAIT_START;
                OS_eActivateTask(APP_ZPR_Light_Task);
            }
            else
            {
                DBG_vPrintf(TRACE_APP,"Event ignored\r\n");
            }
            break;
        }

        case E_RESCAN:
            /* restart scan with configured channel mask */
            DBG_vPrintf(TRACE_APP, "Rescanning with %08x\r\n",ZPS_psAplAibGetAib()->apsChannelMask);
            ZPS_eAplZdoDiscoverNetworks(ZPS_psAplAibGetAib()->apsChannelMask);
            eNodeState = E_NETWORK_DISCOVERY;
        break;

        case E_RUNNING:
        {
            vLedControl(0, 0);

            if (FALSE == bBound)
            {
                if (OS_E_SWTIMER_RUNNING
                        != OS_eGetSWTimerStatus(APP_RestartTimer))
                {
                    vToggleLed(1);
                    OS_eStartSWTimer(APP_RestartTimer, LED_TOGGLE_TIME, NULL);
                }
            }
            else
            {
                vLedControl(1, 0);
            }

            if (APP_E_EVENT_BUTTON_UP == sAppEvent.eType
                    && APP_E_BUTTONS_BUTTON_1 == sAppEvent.sButton.u8Button)
            {
                DBG_vPrintf(TRACE_APP, "Allow joining for %ds\r\n",PERMIT_JOINING_TIME);
                ZPS_eAplZdoPermitJoining(PERMIT_JOINING_TIME);
            }
            else if (APP_E_EVENT_BUTTON_UP == sAppEvent.eType &&
                      APP_E_BUTTONS_BUTTON_2 == sAppEvent.sButton.u8Button)
            {
                DBG_vPrintf(TRACE_APP, "Button pressed\r\n");
                if (bBind() == TRUE)
                {
                    eNodeState = E_BIND_WAIT;
                    OS_eActivateTask(APP_ZPR_Light_Task);
                    OS_eStopSWTimer(APP_RestartTimer);
                    OS_eStartSWTimer(APP_RestartTimer,
                                     BIND_RESPONSE_WAIT_TIMER,
                                     NULL);
                }
            }
            break;
        }

        case E_BIND_WAIT:
        {
            vLedControl(0, 0);
            vLedControl(1, 1);

            if (ZPS_EVENT_NONE != sStackEvent.eType)
            {
                if (ZPS_EVENT_APS_ZDP_REQUEST_RESPONSE == sStackEvent.eType)
    			{
    				if (ED_BIND_RESP == sStackEvent.uEvent.sApsZdpEvent.u16ClusterId)
    				{
    					if(sStackEvent.uEvent.sApsZdpEvent.uZdpData.sBindRsp.u8Status !=  ZPS_E_SUCCESS)
    					{
                            DBG_vPrintf(TRACE_APP, "Binding Failed\r\n");
                        }
                        else
                        {
                            DBG_vPrintf(TRACE_APP, "Binding Response OK\r\n");
                            bBound = TRUE;
                        }

                        eNodeState = E_RUNNING;
                        OS_eStopSWTimer(APP_RestartTimer);
                        OS_eActivateTask(APP_ZPR_Light_Task);
                    }
                    else
                    {
                        DBG_vPrintf(TRACE_APP,"ZDP Event ignored, cluster ID: 0x%x\r\n", sStackEvent.uEvent.sApsZdpEvent.u16ClusterId);
                    }
                }
                else
                {
                    DBG_vPrintf(TRACE_APP,"Event ignored\r\n");
                }
            }
            else if (OS_E_SWTIMER_RUNNING
                    != OS_eGetSWTimerStatus(APP_RestartTimer))
            {
                DBG_vPrintf(TRACE_APP, "No bind response\r\n");
                eNodeState = E_RUNNING;
                OS_eActivateTask(APP_ZPR_Light_Task);
            }
            else
            {
                DBG_vPrintf(TRACE_APP,"Event ignored\r\n");
            }
        }
        break;

    }

    /*
     * Global clean up to make sure any APDUs have been freed
     */
    if (ZPS_EVENT_APS_DATA_INDICATION == sStackEvent.eType)
    {
        PDUM_eAPduFreeAPduInstance(sStackEvent.uEvent.sApsDataIndEvent.hAPduInst);
    }
}

/****************************************************************************/
/***        Local Functions                                               ***/
/****************************************************************************/

/**
 * Called to send the end device bind request.
 */
PRIVATE bool_t bBind(void)
{
    bool_t bStatus = FALSE;

    PDUM_thAPduInstance hAPduInst =
            PDUM_hAPduAllocateAPduInstance(apduApplication);

    if (PDUM_INVALID_HANDLE != hAPduInst)
    {
        ZPS_tsAplZdpEndDeviceBindReq sEndDeviceBindReq;

        uint16 au16InClusters[] =
        { HOME_CONTROLS_ON_OFF_SWITCH_ACTIONS_CLUSTER_ID };

        sEndDeviceBindReq.u16BindingTarget
                = ZPS_u16NwkNibGetNwkAddr(ZPS_pvAplZdoGetNwkHandle());
        sEndDeviceBindReq.u64SrcIeeeAddress
                = ZPS_u64AplZdoGetIeeeAddr();

        DBG_vPrintf(TRACE_APP,"Target MAC:0x%llx Nwk:%d\r\n",
                sEndDeviceBindReq.u64SrcIeeeAddress,
                sEndDeviceBindReq.u16BindingTarget);

        sEndDeviceBindReq.u8SrcEndpoint = 1;
        sEndDeviceBindReq.u16ProfileId = HOME_CONTROLS_PROFILE_ID;
        sEndDeviceBindReq.u8NumInClusters = 1;
        sEndDeviceBindReq.pu16InClusterList = au16InClusters;
        sEndDeviceBindReq.u8NumOutClusters = 0;
        sEndDeviceBindReq.pu16OutClusterList = NULL;

        ZPS_teStatus teStatus =
                ZPS_eAplZdpEndDeviceBindRequest(hAPduInst,
                                                &u8BindingSeqNum,
                                                &sEndDeviceBindReq);

        if (teStatus)
        {
            DBG_vPrintf(TRACE_APP, "Error, Bind Failed: %x\r\n", teStatus );
        }
        else
        {
            DBG_vPrintf(TRACE_APP, "Binding Request OK, seq num %d\r\n", u8BindingSeqNum );
            bStatus = TRUE;
        }
    }
    else
    {
        DBG_vPrintf(TRACE_APP, "Error, no free PDUs\r\n");
    }

    return bStatus;
}

/**
 * Toggles the specified LED
 */
PRIVATE void vToggleLed(uint8 u8LedNum)
{
    static bool_t bLedStates[2] = { FALSE, FALSE };

    bLedStates[u8LedNum] = !bLedStates[u8LedNum];
    vLedControl(u8LedNum,bLedStates[u8LedNum]);
}

/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
